@@ -56,6 +56,11 @@ extern void make_trace_window();
 	(str2hex(cmd.c_str(), cmd.length())), \
 	(str2hex(replystr.c_str(), replystr.length())) )
 
+/*
 #define geth() get_trace(2, \
 	(str2hex(cmd.c_str(), cmd.length())), \
 	(str2hex(replystr.c_str(), replystr.length())) )
+*/
+
+#define geth() get_trace(1, \
+	(str2hex(replystr.c_str(), replystr.length())) )
