@@ -81,6 +81,7 @@
 #include "icons.h"
 #include "cwio.h"
 #include "cwioUI.h"
+#include "serial.h"
 
 #include "flrig_icon.cxx"
 
@@ -254,6 +255,9 @@ void exit_main(Fl_Widget *w)
 	if (Fl::event_key() == FL_Escape)
 		return;
 	cbExit();
+#if SERIAL_DEBUG
+	fclose(serlog);
+#endif
 }
 
 void expand_controls(void*)
@@ -268,16 +272,12 @@ void close_controls(void*)
 			if (EXPAND_CONTROLS && selrig->has_extras) return;
 			btn_show_controls->label("@-22->");
 			btn_show_controls->redraw_label();
-//			grpTABS->hide();
-//			mainwindow->resizable(grpTABS);
 			mainwindow->size(progStatus.mainW, 150);
 			mainwindow->size_range(735, 150, 0, 150);
 			if (tabs_dialog && tabs_dialog->visible()) tabs_dialog->hide();
 			break;
 		case small_ui :
 			if (tabs_dialog && tabs_dialog->visible()) tabs_dialog->hide();
-//			if (EXPAND_CONTROLS && selrig->has_extras)
-//				Fl::add_timeout(1.0, expand_controls);
 			break;
 		case touch_ui :
 		default :
@@ -311,22 +311,24 @@ void startup(void*)
 
 void rotate_log(std::string filename)
 {
-	const int n = 5; // rename existing log files to keep up to 5 old versions
-	ostringstream oldfn, newfn;
-	ostringstream::streampos p;
+	std::string f1, f2, f3, f4, f5;
 
-	oldfn << filename << '.';
-	newfn << filename << '.';
-	p = oldfn.tellp();
+	f1.assign(filename).append(".1");
+	f2.assign(filename).append(".2");
+	f3.assign(filename).append(".3");
+	f4.assign(filename).append(".4");
+	f5.assign(filename).append(".5");
+
+	rename(f4.c_str(), f5.c_str());
+	rename(f3.c_str(), f4.c_str());
+	rename(f2.c_str(), f3.c_str());
+	rename(f1.c_str(), f2.c_str());
+	rename(filename.c_str(), f1.c_str());
+
+	ofstream tfile(filename.c_str());
+	tfile << "flrig " << VERSION << std::endl << std::endl;
+	tfile.close();
 
-	for (int i = n - 1; i > 0; i--) {
-		oldfn.seekp(p);
-		newfn.seekp(p);
-		oldfn << i;
-		newfn << i + 1;
-		rename(oldfn.str().c_str(), newfn.str().c_str());
-	}
-	rename(filename.c_str(), oldfn.str().c_str());
 }
 
 void flrig_terminate() {
@@ -388,7 +390,6 @@ int main (int argc, char *argv[])
 	checkdirectories();
 
 #if SERIAL_DEBUG
-extern FILE *serlog;
 	std::string serlogname = RigHomeDir;
 	serlogname.append("serlog.txt");
 	serlog = fopen(serlogname.c_str(), "w");
@@ -444,6 +445,7 @@ extern FILE *serlog;
 	cwio_keyer_dialog = cwio_window();
 	cwio_editor = make_message_editor();
 	cwio_configure = cwio_config_dialog();
+	morse->init();
 
 	fntbrowser = new Font_Browser;
 	dlgMemoryDialog = Memory_Dialog();
