@@ -62,12 +62,14 @@ Fl_Button*			btn_cleartrace = (Fl_Button*)0;
 
 vector<string> tracestrings;
 
+bool stdout_trace = false;
+
 static void cb_cleartrace(Fl_Button *, void *)
 {
 	tracedisplay->buffer()->text("");
 }
 
-static void make_trace_window() {
+void make_trace_window() {
 	tracewindow = new Fl_Double_Window(600, 300, _("Trace log"));
 	tracedisplay = new Fl_Text_Display(0, 0, 600, 270);
 	tracebuffer = new Fl_Text_Buffer;
@@ -79,19 +81,20 @@ static void make_trace_window() {
 	tracewindow->resizable(tracedisplay);
 }
 
+static void write_trace_file(std::string s)
+{
+	string trace_fname = RigHomeDir;
+	trace_fname.append("trace.txt");
+	fstream tfile(trace_fname.c_str(), ios::app);
+	if (tfile)
+		tfile << s;
+	tfile.close();
+}
+
 static void update_tracetext(void *)
 {
 	guard_lock tt(&mutex_trace);
 
-	string trace_fname = RigHomeDir;
-	trace_fname.append("trace.txt");
-	ofstream tfile(trace_fname.c_str(), ios::app);
-	if (tfile)  {
-		for (size_t n = 0; n < tracestrings.size(); n++)
-			tfile << tracestrings[n];
-	}
-	tfile.close();
-
 	if (tracewindow->visible()) {
 		if (tracedisplay->buffer()->length() > 100000)
 			tracedisplay->buffer()->text("");
@@ -103,11 +106,12 @@ static void update_tracetext(void *)
 
 void trace(int n, ...) // all args of type const char *
 {
-	if (!tracewindow) make_trace_window();
-
 	if (!progStatus.trace) return;
 
+	if (!tracewindow) make_trace_window();
+
 	if (!n) return;
+
 	stringstream s;
 	va_list vl;
 	va_start(vl, n);
@@ -117,18 +121,27 @@ void trace(int n, ...) // all args of type const char *
 	va_end(vl);
 	s << "\n";
 
+	if (stdout_trace) {
+		std::cout << s.str();
+		return;
+	}
+
+	write_trace_file(s.str());
+
 	guard_lock tt(&mutex_trace);
-		tracestrings.push_back(s.str());
-		Fl::awake(update_tracetext);
+
+	tracestrings.push_back(s.str());
+	Fl::awake(update_tracetext);
 }
 
 void xml_trace(int n, ...) // all args of type const char *
 {
-	if (!tracewindow) make_trace_window();
-
 	if (!progStatus.xmltrace) return;
 
+	if (!tracewindow) make_trace_window();
+
 	if (!n) return;
+
 	stringstream s;
 	va_list vl;
 	va_start(vl, n);
@@ -138,18 +151,21 @@ void xml_trace(int n, ...) // all args of type const char *
 	va_end(vl);
 	s << "\n";
 
+	if (stdout_trace) {
+		std::cout << s.str();
+		return;
+	}
+
+	write_trace_file(s.str());
+
 	guard_lock tt(&mutex_trace);
-		tracestrings.push_back(s.str());
-		Fl::awake(update_tracetext);
+
+	tracestrings.push_back(s.str());
+	Fl::awake(update_tracetext);
 }
 
 void rig_trace(int n, ...) // all args of type const char *
 {
-	if (!tracewindow) make_trace_window();
-
-	if (!progStatus.rigtrace) return;
-
-	if (!n) return;
 	stringstream s;
 	va_list vl;
 	va_start(vl, n);
@@ -159,18 +175,25 @@ void rig_trace(int n, ...) // all args of type const char *
 	va_end(vl);
 	s << "\n";
 
+	if (stdout_trace) {
+		std::cout << s.str();
+		return;
+	}
+
+	if (!progStatus.rigtrace) return;
+	if (!tracewindow) make_trace_window();
+	if (!n) return;
+
+	write_trace_file(s.str());
+
 	guard_lock tt(&mutex_trace);
-		tracestrings.push_back(s.str());
-		Fl::awake(update_tracetext);
+
+	tracestrings.push_back(s.str());
+	Fl::awake(update_tracetext);
 }
 
 void set_trace(int n, ...) // all args of type const char *
 {
-	if (!tracewindow) make_trace_window();
-
-	if (!progStatus.settrace) return;
-
-	if (!n) return;
 	stringstream s;
 	va_list vl;
 	va_start(vl, n);
@@ -180,18 +203,25 @@ void set_trace(int n, ...) // all args of type const char *
 	va_end(vl);
 	s << "\n";
 
+	if (stdout_trace) {
+		std::cout << s.str();
+		return;
+	}
+
+	if (!progStatus.settrace) return;
+	if (!tracewindow) make_trace_window();
+	if (!n) return;
+
+	write_trace_file(s.str());
+
 	guard_lock tt(&mutex_trace);
-		tracestrings.push_back(s.str());
-		Fl::awake(update_tracetext);
+
+	tracestrings.push_back(s.str());
+	Fl::awake(update_tracetext);
 }
 
 void get_trace(int n, ...) // all args of type const char *
 {
-	if (!tracewindow) make_trace_window();
-
-	if (!progStatus.gettrace) return;
-
-	if (!n) return;
 	stringstream s;
 	va_list vl;
 	va_start(vl, n);
@@ -201,18 +231,26 @@ void get_trace(int n, ...) // all args of type const char *
 	va_end(vl);
 	s << "\n";
 
+	if (stdout_trace) {
+		std::cout << s.str();
+		return;
+	}
+
+	if (!progStatus.gettrace) return;
+	if (!tracewindow) make_trace_window();
+	if (!n) return;
+
+	write_trace_file(s.str());
+
 	guard_lock tt(&mutex_trace);
-		tracestrings.push_back(s.str());
-		Fl::awake(update_tracetext);
+
+	tracestrings.push_back(s.str());
+
+	Fl::awake(update_tracetext);
 }
 
 void rpc_trace(int n, ...) // all args of type const char *
 {
-	if (!tracewindow) make_trace_window();
-
-	if (!progStatus.rpctrace) return;
-
-	if (!n) return;
 	stringstream s;
 	va_list vl;
 	va_start(vl, n);
@@ -262,8 +300,47 @@ void rpc_trace(int n, ...) // all args of type const char *
 
 	if (str[str.length()-1] != '\n') str += '\n';
 
+	if (stdout_trace) {
+		std::cout << s.str();
+		return;
+	}
+
+	if (!tracewindow) make_trace_window();
+	if (!progStatus.rpctrace) return;
+	if (!n) return;
+
+	write_trace_file(s.str());
+
 	guard_lock tt(&mutex_trace);
-		tracestrings.push_back(str);
-		Fl::awake(update_tracetext);
+
+	tracestrings.push_back(str);
+	Fl::awake(update_tracetext);
 }
 
+void ser_trace(int n, ...) // all args of type const char *
+{
+	stringstream s;
+	va_list vl;
+	va_start(vl, n);
+	s << ztime() << " : " << va_arg(vl, const char *);
+	for (int i = 1; i < n; i++)
+		s << " " << va_arg(vl, const char *);
+	va_end(vl);
+	s << "\n";
+
+	if (stdout_trace) {
+		std::cout << s.str();
+		return;
+	}
+
+	if (!progStatus.serialtrace) return;
+	if (!tracewindow) make_trace_window();
+	if (!n) return;
+
+	write_trace_file(s.str());
+
+	guard_lock tt(&mutex_trace);
+
+	tracestrings.push_back(s.str());
+	Fl::awake(update_tracetext);
+}
